#include <iostream.h>
#include "segment.hh"


SegmentMemory::SegmentMemory(MemoryArea** pma,unsigned int nma)
  : MemoryArea(SEGMENT_MEMORY), size(0), data_size(0)
{
  for(int i=0;i<nma;i++)
    {
      (pma[i])->AddUse();
      size      += (pma[i])->Size();
      data_size += (pma[i])->DataSize();
      areas_list.AddTail(pma[i]);
    }
}

SegmentMemory::SegmentMemory(FlatMemory& _ma)
  : MemoryArea(SEGMENT_MEMORY), size(_ma.Size()), data_size(_ma.DataSize())
{
  _ma.AddUse();
  areas_list.AddHead(&_ma);
}

SegmentMemory::SegmentMemory(SegmentMemory& _sg)
  : MemoryArea(SEGMENT_MEMORY), size(_sg.size), data_size(_sg.data_size)
{
  for(int i=1; i<=_sg.areas_list.Length();i++)
    {
      (_sg.areas_list[i]).AddUse();
      areas_list.AddTail(&(_sg.areas_list[i]));
    }
}

SegmentMemory::~SegmentMemory()
{
  //cout << "SegmentMemory: ~SegmentMemory() begin\n"; 

  MemoryArea*  pma;
  unsigned int steps = areas_list.Length();  

  while(steps > 0)
    {
      //cout << "SegmentMemory: removing memory\n";

      if((pma = areas_list.RemHead()) == NULL)
	{
	  cerr << "SegmentMemory: find a NULL memory while destroing segments\n";
	}
      else	
	if(pma->Use() == 0)
	  {
	    cerr << "SegmentMemory: use count is 0. BUG! BUG! BUG!\n";
	    ;
	  }
	else if(pma->Use() == 1)
	  {
	    cerr << "SegmentMemory: use count is 1. BUG! BUG! BUG!\n";
	    ;
	  }
	else 
	  {
	    pma->RemUse();
	  }
      steps--;
    }

  //cout << "SegmentMemory: ~SegmentMemory() end\n"; 
}

unsigned int SegmentMemory::Read(void* pdata,unsigned int length,unsigned int offset)
{
  int  i = 0;
  int  toread = 0;
  int  ma_offset = 0;
  int  readed = 0;

  if(offset >= data_size)
    return 0;

  if(length > (data_size - offset))
    length = data_size - offset;

  i = 1;
  while(1)
    {
      if(offset < areas_list[i].DataSize())
	break;
      else
	{
	  offset -= areas_list[i].DataSize();
	  i++;
	}
    }

  toread = length;
  ma_offset = 0;
  readed = areas_list[i].Read(pdata,toread,offset);
  ma_offset += readed;
  toread -= readed;
  i++;

  while(toread > 0)
    {
      readed = areas_list[i].Read(pdata + ma_offset,toread,0);
      ma_offset += readed;
      toread -= readed;
      i++;
    }

  return length;
}

unsigned int SegmentMemory::Write(void* pdata,unsigned int length,unsigned int offset)
{
  int  i = 0;
  int  towrite = 0;
  int  ma_offset = 0;
  int  writed = 0;

  if(offset >= size)
    return 0;

  if(length > (size - offset))
    length = size - offset;

  data_size = length + offset;

  i = 1;
  while(1)
    {
      if(offset <= areas_list[i].DataSize())
	break;
      else
	{
	  offset -= areas_list[i].DataSize();
	  i++;
	}
    }

  towrite = length;
  ma_offset = 0;
  writed = areas_list[i].Write(pdata,towrite,offset);
  ma_offset += writed;
  towrite -= writed;
  i++;

  while(towrite > 0)
    {
      writed = areas_list[i].Write(pdata + ma_offset,towrite,0);
      ma_offset += writed;
      towrite -= writed;
      i++;
    }

  return length;
}

unsigned int SegmentMemory::Size()
{
  return size;
}

unsigned int SegmentMemory::DataSize()
{
  return data_size;
}

FlatMemory* SegmentMemory::FirstFlat()
{
  if(areas_list[1].Type() == FLAT_MEMORY)
    return (FlatMemory*)&(areas_list[1]);
  else
    return ((SegmentMemory*)&(areas_list[1]))->FirstFlat();
}

/*
void Message::Prepend(MemoryArea& ma)
{
  areas_list.Cons(&ma);
  data_size += ma.DataSize();
}
*/
